@@ -11,7 +11,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
 
 from keystoneauth1 import exceptions as ks_exc
 from keystoneauth1 import loading as keystone
@@ -26,6 +26,7 @@ from neutron_lib.tests import _base as base
 
 
 RESOURCE_PROVIDER_UUID = uuidutils.generate_uuid()
+CONSUMER_UUID = uuidutils.generate_uuid()
 RESOURCE_PROVIDER_NAME = 'resource_provider_name'
 RESOURCE_PROVIDER = {
     'uuid': RESOURCE_PROVIDER_UUID,
@@ -698,3 +699,138 @@ class TestPlacementAPIClient(base.BaseTestCase):
             resource_provider_generation=None,
         )
         self.assertEqual(1, self.placement_fixture.mock_put.call_count)
+
+    def test_list_allocations(self):
+        self.placement_api_client.list_allocations(CONSUMER_UUID)
+        self.placement_fixture.mock_get.assert_called_once_with(
+            '/allocations/%s' % CONSUMER_UUID)
+
+    def test_update_allocation(self):
+        mock_rsp = mock.Mock()
+        mock_rsp.json = lambda: {
+            'allocations': {
+                RESOURCE_PROVIDER_UUID: {'resources': {'a': 10}}
+            }
+        }
+        self.placement_fixture.mock_get.side_effect = [mock_rsp]
+        self.placement_api_client.update_allocation(
+            CONSUMER_UUID,
+            {'allocations': {
+                RESOURCE_PROVIDER_UUID: {
+                    'resources': {'a': 20}}
+            }})
+        self.placement_fixture.mock_put.assert_called_once_with(
+            '/allocations/%s' % CONSUMER_UUID,
+            {'allocations': {
+                RESOURCE_PROVIDER_UUID: {
+                    'resources': {'a': 20}}
+            }}
+        )
+
+    def test_update_qos_minbw_allocation(self):
+        mock_rsp_get = mock.Mock()
+        mock_rsp_get.json = lambda: {
+            'allocations': {
+                RESOURCE_PROVIDER_UUID: {
+                    'resources': {'a': 3, 'b': 2}
+                }
+            }
+        }
+        self.placement_fixture.mock_get.side_effect = [mock_rsp_get]
+        self.placement_api_client.update_qos_minbw_allocation(
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={'a': 2, 'b': 2},
+            rp_uuid=RESOURCE_PROVIDER_UUID
+        )
+        self.placement_fixture.mock_put.assert_called_once_with(
+            '/allocations/%s' % CONSUMER_UUID,
+            {'allocations': {
+                RESOURCE_PROVIDER_UUID: {
+                    'resources': {'a': 5, 'b': 4}}
+            }}
+        )
+
+    def test_update_qos_minbw_allocation_removed(self):
+        mock_rsp = mock.Mock()
+        mock_rsp.json = lambda: {'allocations': {}}
+        self.placement_fixture.mock_get.side_effect = [mock_rsp]
+        self.assertRaises(
+            n_exc.PlacementAllocationRemoved,
+            self.placement_api_client.update_qos_minbw_allocation,
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={'a': 1, 'b': 1},
+            rp_uuid=RESOURCE_PROVIDER_UUID
+        )
+
+    def test_update_qos_minbw_allocation_rp_not_exists(self):
+        mock_rsp = mock.Mock()
+        mock_rsp.json = lambda: {'allocations': {'other:rp:uuid': {'c': 3}}}
+        self.placement_fixture.mock_get.side_effect = [mock_rsp]
+        self.assertRaises(
+            n_exc.PlacementAllocationRpNotExists,
+            self.placement_api_client.update_qos_minbw_allocation,
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={'a': 1, 'b': 1},
+            rp_uuid=RESOURCE_PROVIDER_UUID
+        )
+
+    def test_update_qos_minbw_allocation_max_retries(self):
+        mock_rsp_get = mock.Mock()
+        mock_rsp_get.json = lambda: {
+            'allocations': {RESOURCE_PROVIDER_UUID: {'c': 3}}
+        }
+        self.placement_fixture.mock_get.side_effect = 10 * [mock_rsp_get]
+        mock_rsp_put = mock.Mock()
+        mock_rsp_put.json = lambda: {
+            'errors': [{'code': 'placement.concurrent_update'}]}
+        self.placement_fixture.mock_put.side_effect = ks_exc.Conflict(
+            response=mock_rsp_put)
+        self.assertRaises(
+            n_exc.PlacementAllocationGenerationConflict,
+            self.placement_api_client.update_qos_minbw_allocation,
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={},
+            rp_uuid=RESOURCE_PROVIDER_UUID,
+        )
+        self.assertEqual(10, self.placement_fixture.mock_put.call_count)
+
+    def test_update_qos_minbwallocation_generation_conflict_solved(self):
+        mock_rsp_get = mock.Mock()
+        mock_rsp_get.json = lambda: {
+            'allocations': {RESOURCE_PROVIDER_UUID: {'c': 3}}
+        }
+        self.placement_fixture.mock_get.side_effect = 2 * [mock_rsp_get]
+        mock_rsp_put = mock.Mock()
+        mock_rsp_put.json = lambda: {
+            'errors': [{'code': 'placement.concurrent_update'}]}
+        self.placement_fixture.mock_put.side_effect = [
+            ks_exc.Conflict(response=mock_rsp_put),
+            mock.Mock()
+        ]
+        self.placement_api_client.update_qos_minbw_allocation(
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={},
+            rp_uuid=RESOURCE_PROVIDER_UUID
+        )
+        self.assertEqual(2, self.placement_fixture.mock_put.call_count)
+
+    def test_update_qos_minbw_allocation_other_conflict(self):
+        mock_rsp_get = mock.Mock()
+        mock_rsp_get.json = lambda: {
+            'allocations': {RESOURCE_PROVIDER_UUID: {'c': 3}}
+        }
+        self.placement_fixture.mock_get.side_effect = 10*[mock_rsp_get]
+        mock_rsp_put = mock.Mock()
+        mock_rsp_put.text = ''
+        mock_rsp_put.json = lambda: {
+            'errors': [{'code': 'some other error code'}]}
+        self.placement_fixture.mock_put.side_effect = ks_exc.Conflict(
+            response=mock_rsp_put)
+        self.assertRaises(
+            ks_exc.Conflict,
+            self.placement_api_client.update_qos_minbw_allocation,
+            consumer_uuid=CONSUMER_UUID,
+            minbw_alloc_diff={},
+            rp_uuid=RESOURCE_PROVIDER_UUID,
+        )
+        self.placement_fixture.mock_put.assert_called_once()
